/*
 * Copyright (C), 2020-2024, KylinSoft. Co., Ltd.
 * @Author: zhangfan
 * @FileName: tpm_security.h
 * @Date: 2024-06-14 14:01:20
 * @Version: v1.0
 * @LastEditors: zhangfan4 zhangfan4@kylinos.cn
 * @LastEditTime: 2024-08-29 14:57:12
 * @Description: TPM安全相关的头文件
 * @History: Reference changelog
 * @Function List: 
 */
#ifndef __TPM_SECURITY_H__
#define __TPM_SECURITY_H__

#include <err.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdint.h>
#include <stdbool.h>

struct TPMAlgorithm {
	char alg[16];
};

struct TPMSupportAlgorithm {
	uint32_t count;
	struct TPMAlgorithm  algs[128];
};

struct TPMTrustedRootData {
	char chipType[8]; //可信根类型
	char chipVersion[8]; //芯片版本号
	char specVersion[8]; //规范版本号
	char manufacturer[4];  //厂商
	char chipname[128]; //芯片名称
	uint32_t nvDefined; //已定义NV个数
	char supportAlg[1024]; //支持算法
};


struct TPMBootMeasureData{
	char time[64];    //度量时间
	char pname[256]; //度量进程名
	char measure_result[10];
	int start_measure_error_number;    
};

struct TPMBootMeasureLog {
	uint32_t numbers;
	struct TPMBootMeasureData report[32];
} ;

struct TPM_TrustChainData {
	char bios[16]; //bios阶段度量结果
	char grub[16]; //grub模块度量结果
	char kernel[16]; //kernel模块度量结果
	char time[64]; //时间
	int  totalnum; //度量文件总数
	int  failnum; //度量失败文件总数
};


typedef enum 
{
	tpm_tpcm = 0, //TPCM阶段（TPM/TCM可信根不涉及此阶段）
	tpm_uefi = 1, //UEFI阶段（TPM/TCM可信根不涉及此阶段）
	tpm_grub = 2, //Grub阶段
	tpm_bios = 3  //BIOS阶段
}TpmReportOpt;

#ifdef __cplusplus
extern "C" {
#endif


	int TPM_GetTrustedRootData(struct TPMTrustedRootData *data, int nvpasswd_len, char * nvpasswd);
	int TPM_GetBootMeasureStatus();
	int TPM_GetBootMeasureCfgStatus();
	int TPM_SetBootMeasureCfgStatus(int status, char *devicepath, int *trustrootmode, int nvpasswd_len, char * nvpasswd, int authorization_len, char * authorization_para);
	int TPM_SetReCollectMeasureStandarValue(char *filepath, int nvpasswd_len, char * nvpasswd, int authorization_len, char * authorization_para);
	int TPM_GetReCollectMeasureStandarValue();
	int TPM_GetBootMeasureData(int reportopt, struct TPMBootMeasureData **list, int *count);
	int TPM_GetHistoryBootMeasureLog(struct TPMBootMeasureLog **list, int *count);

	int TPM_GetBootMode();
	int TPM_SetBootMode(int status);
	int TPM_SaveFlashBootMeasureLog(); 
	int TPM_AddFile(char *filepath, int nvpasswd_len, char * nvpasswd, int authorization_len, char * authorization_para);
	int TPM_DealFile(char *filepath, int nvpasswd_len, char * nvpasswd, int authorization_len, char * authorization_para);
	int TPM_GetTrustChainData(struct TPM_TrustChainData* data);




#ifdef __cplusplus
}
#endif


#endif //__TPM_SECURITY_H__
